@@ -19,83 +19,84 @@ pca.fd <- function(fdobj, nharm = 2, harmfdPar=fdPar(fdobj),
 #  meanfd     ... A functional data object giving the mean function
 #
 
-#  Last modified:  24 April 2012 by Jim Ramsay
+#  Last modified:  28 December 2012 by Jim Ramsay
 
-  #  Check FDOBJ
+#  Check FDOBJ
 
-  if (!(inherits(fdobj, "fd"))) stop(
+if (!(inherits(fdobj, "fd"))) stop(
     "Argument FD  not a functional data object.")
 
-  #  compute mean function and center if required
+#  compute mean function and center if required
 
-  meanfd <- mean.fd(fdobj)
-  if (centerfns) fdobj <- center.fd(fdobj)
+if (centerfns) {
+  fdobj <- center.fd(fdobj)
+}
 
-  #  get coefficient matrix and its dimensions
+#  get coefficient matrix and its dimensions
 
-  coef  <- fdobj$coefs
-  coefd <- dim(coef)
-  ndim  <- length(coefd)
-  nrep  <- coefd[2]
-  coefnames <- dimnames(coef)
-  if (nrep < 2) stop("PCA not possible without replications.")
+coef  <- fdobj$coefs
+coefd <- dim(coef)
+ndim  <- length(coefd)
+nrep  <- coefd[2]
+coefnames <- dimnames(coef)
+if (nrep < 2) stop("PCA not possible without replications.")
 
-  basisobj <- fdobj$basis
-  nbasis   <- basisobj$nbasis
-  type     <- basisobj$type
+basisobj <- fdobj$basis
+nbasis   <- basisobj$nbasis
+type     <- basisobj$type
 
-  #  set up HARMBASIS
+#  set up HARMBASIS
 
-  harmbasis <- harmfdPar$fd$basis
-  nhbasis   <- harmbasis$nbasis
+harmbasis <- harmfdPar$fd$basis
+nhbasis   <- harmbasis$nbasis
 
-  #  set up LFDOBJ and LAMBDA
+#  set up LFDOBJ and LAMBDA
 
-  Lfdobj <- harmfdPar$Lfd
-  lambda <- harmfdPar$lambda
+Lfdobj <- harmfdPar$Lfd
+lambda <- harmfdPar$lambda
 
-  #  compute CTEMP whose cross product is needed
+#  compute CTEMP whose cross product is needed
 
-  if (ndim == 3) {
-    nvar <- coefd[3]
-    ctemp <- matrix(0, nvar * nbasis, nrep)
-    for(j in 1:nvar) {
+if (ndim == 3) {
+  nvar <- coefd[3]
+  ctemp <- matrix(0, nvar * nbasis, nrep)
+  for(j in 1:nvar) {
       index <- 1:nbasis + (j - 1) * nbasis
       ctemp[index,  ] <- coef[,  , j]
-    }
-  } else {
+  }
+} else {
     nvar  <- 1
     ctemp <- coef
-  }
+}
 
-  #  set up cross product Lmat for harmonic basis,
-  #  roughness penalty matrix Rmat, and
-  #  penalized cross product matrix Lmat
+#  set up cross product Lmat for harmonic basis,
+#  roughness penalty matrix Rmat, and
+#  penalized cross product matrix Lmat
 
-  Lmat <- eval.penalty(harmbasis, 0)
-  if (lambda > 0) {
+Lmat <- eval.penalty(harmbasis, 0)
+if (lambda > 0) {
     Rmat <- eval.penalty(harmbasis, Lfdobj)
     Lmat <- Lmat + lambda * Rmat
-  }
-  Lmat <- (Lmat + t(Lmat))/2
+}
+Lmat <- (Lmat + t(Lmat))/2
 
-  #  compute the Choleski factor of Wmat
+#  compute the Choleski factor Mmat of Lmat
 
-  Mmat    <- chol(Lmat)
-  Mmatinv <- solve(Mmat)
+Mmat    <- chol(Lmat)
+Mmatinv <- solve(Mmat)
 
-    #  set up cross product and penalty matrices
+#  set up cross product and penalty matrices
 
-  Wmat <- crossprod(t(ctemp))/nrep
+Wmat <- crossprod(t(ctemp))/nrep
   
-  Jmat = inprod(harmbasis, basisobj)
-  MIJW = crossprod(Mmatinv,Jmat)
+Jmat = inprod(harmbasis, basisobj)
+MIJW = crossprod(Mmatinv,Jmat)
   
-  #  set up matrix for eigenanalysis
+#  set up matrix for eigenanalysis
 
-  if(nvar == 1) {
+if(nvar == 1) {
     Cmat = MIJW %*% Wmat %*% t(MIJW)
-  } else {
+} else {
     Cmat = matrix(0,nvar*nhbasis,nvar*nhbasis)
     for (i in 1:nvar) {
       indexi <- 1:nbasis + (i - 1) * nbasis
@@ -104,45 +105,45 @@ pca.fd <- function(fdobj, nharm = 2, harmfdPar=fdPar(fdobj),
         Cmat[indexi, indexj] <- MIJW %*% Wmat[indexi,indexj] %*% t(MIJW)
       }
     }
-  }
+}
 
-  #  eigenalysis
+#  eigenalysis
 
-  Cmat    <- (Cmat + t(Cmat))/2
-  result  <- eigen(Cmat)
-  eigvalc <- result$values
-  eigvecc <- as.matrix(result$vectors[, 1:nharm])
-  sumvecc <- apply(eigvecc, 2, sum)
-  eigvecc[,sumvecc < 0] <-  - eigvecc[, sumvecc < 0]
+Cmat    <- (Cmat + t(Cmat))/2
+result  <- eigen(Cmat)
+eigvalc <- result$values
+eigvecc <- as.matrix(result$vectors[, 1:nharm])
+sumvecc <- apply(eigvecc, 2, sum)
+eigvecc[,sumvecc < 0] <-  - eigvecc[, sumvecc < 0]
 
-  varprop <- eigvalc[1:nharm]/sum(eigvalc)
+varprop <- eigvalc[1:nharm]/sum(eigvalc)
 
-  #  set up harmfd
+#  set up harmfd
   
-  if (nvar == 1) {
+if (nvar == 1) {
     harmcoef <- Mmatinv %*% eigvecc
-  } else {
+ } else {
     harmcoef <- array(0, c(nbasis, nharm, nvar))
     for (j in 1:nvar) {
       index <- 1:nbasis + (j - 1) * nbasis
       temp <- eigvecc[index,  ]
       harmcoef[,  , j] <- Mmatinv %*% temp
     }
-  }
-  harmnames <- rep("", nharm)
-  for(i in 1:nharm)
+}
+harmnames <- rep("", nharm)
+for(i in 1:nharm)
     harmnames[i] <- paste("PC", i, sep = "")
-  if(length(coefd) == 2)
+if(length(coefd) == 2)
     harmnames <- list(coefnames[[1]], harmnames,"values")
-  if(length(coefd) == 3)
+if(length(coefd) == 3)
     harmnames <- list(coefnames[[1]], harmnames, coefnames[[3]])
-  harmfd   <- fd(harmcoef, harmbasis, harmnames)
+harmfd   <- fd(harmcoef, harmbasis, harmnames)
 
-  #  set up harmscr
+#  set up harmscr
   
-  if (nvar == 1) {
+if (nvar == 1) {
     harmscr  <- inprod(fdobj, harmfd)
-  } else {
+} else {
     harmscr  <- array(0, c(nrep,   nharm, nvar))
     coefarray <- fdobj$coefs
     harmcoefarray <- harmfd$coefs
@@ -151,13 +152,14 @@ pca.fd <- function(fdobj, nharm = 2, harmfdPar=fdPar(fdobj),
       harmfdj <- fd(as.matrix(harmcoefarray[,,j]), basisobj)
       harmscr[,,j] <- inprod(fdobjj, harmfdj)
     }
-  }
+}
 
-  #  set up the object pcafd of the pca.fd class containing the results
+#  set up the object pcafd of the pca.fd class containing the results
   
-  pcafd        <- list(harmfd, eigvalc, harmscr, varprop, meanfd)
-  class(pcafd) <- "pca.fd"
-  names(pcafd) <- c("harmonics", "values", "scores", "varprop", "meanfd")
+meanfd <- mean.fd(fdobj)
+pcafd        <- list(harmfd, eigvalc, harmscr, varprop, meanfd)
+class(pcafd) <- "pca.fd"
+names(pcafd) <- c("harmonics", "values", "scores", "varprop", "meanfd")
 
-  return(pcafd)
+return(pcafd)
 }
