@@ -1,6 +1,7 @@
 smooth.monotone <- function(argvals, y, WfdParobj, wtvec=rep(1,n),
                             zmat=NULL, conv=.0001, iterlim=50,
-                            active=rep(TRUE,nbasis), dbglev=1)
+                            active=rep(TRUE,nbasis), dbglev=1,
+                            returnMatrix=FALSE)
 {
 #  Smooths the relationship of Y to ARGVALS using weights in WTVEC by
 #  fitting a monotone function of the form
@@ -57,6 +58,9 @@ smooth.monotone <- function(argvals, y, WfdParobj, wtvec=rep(1,n),
 #  DBGLEV  ...  Controls the level of output on each iteration.  If 0,
 #               no output, if 1, output at each iteration, if higher,
 #               output at each line search iteration. 1 by default.
+#  RETURNMATRIX ... If False, a matrix in sparse storage model can be returned
+#               from a call to function BsplineS.  See this function for
+#               enabling this option.
 
 #  Returns are:
 #  WFD     ...  Functional data object for W.
@@ -91,8 +95,7 @@ smooth.monotone <- function(argvals, y, WfdParobj, wtvec=rep(1,n),
 #  FLIST and Y2CMAP objects are indexed linear with curves varying
 #  inside variables.
 
-# last modified 2008.11.22 by Spencer Graves
-# previously modified 26 September 2008 by Jim Ramsay
+# last modified 11 May 2012 by Spencer Graves
 
 #  check ARGVALS
 
@@ -147,7 +150,7 @@ if (!is.null(zmat)) {
 ncovp1 <- ncov + 1  #  index for regression coef. for monotone fn.
 wtroot <- sqrt(wtvec)
 wtrtmt <- wtroot %*% matrix(1,1,ncovp1)
-yroot  <- y*wtroot
+yroot  <- y*as.numeric(wtroot)
 climit <- c(-100*rep(1,nbasis), 100*rep(1,nbasis))
 inact  <- !active   #  indices of inactive coefficients
 
@@ -205,7 +208,7 @@ for (ivar in 1:nvar) {
   #  Compute initial function and gradient values
 
   result <- fngrad.smooth.monotone(yi, argvals, zmat, wtvec, cveci, lambda,
-                                   basisobj, Kmat, inact, basislist)
+                                   basisobj, Kmat, inact, basislist, returnMatrix)
   Flisti <- result[[1]]
   betai  <- result[[2]]
   Dyhat  <- result[[3]]
@@ -334,7 +337,8 @@ for (ivar in 1:nvar) {
         #  compute new function value and gradient
         cvecnew <- cveci + linemat[1,5]*deltac
         result  <- fngrad.smooth.monotone(yi, argvals, zmat, wtvec, cvecnew, lambda,
-                                          basisobj, Kmat, inact, basislist)
+                                          basisobj, Kmat, inact, basislist,
+                                          returnMatrix)
         Flisti  <- result[[1]]
         betai   <- result[[2]]
         Dyhat   <- result[[3]]
@@ -457,14 +461,14 @@ if (is.null(zmat)) {
   rangeval <- basisobj$rangeval
   narg     <- 10*nbasis+1
   evalarg  <- seq(rangeval[1], rangeval[2], len=narg)
-  hmat     <- eval.monfd(evalarg, Wfdobj)
+  hmat     <- eval.monfd(evalarg, Wfdobj, 0, returnMatrix)
   if (ndim == 2) {
     yhatmat <- matrix(0,narg,ncurve)
     for (icurve in 1:ncurve) {
       yhatmat[,icurve] <- beta[1,icurve] +
                           beta[2,icurve]*hmat[,icurve]
     }
-    yhatcoef <- project.basis(yhatmat, evalarg, basisobj) 
+    yhatcoef <- project.basis(yhatmat, evalarg, basisobj)
     yhatfd   <- fd(yhatcoef, basisobj)
   } else {
     yhatcoef <- array(0,c(nbasis,ncurve,nvar))
@@ -508,7 +512,8 @@ linesearch.smooth.monotone <- function(Flisti, hessmat, dbglev)
 #  ----------------------------------------------------------------
 
 fngrad.smooth.monotone <- function(yi, argvals, zmat, wtvec, cveci, lambda,
-                                   basisobj, Kmat, inact, basislist)
+                                   basisobj, Kmat, inact, basislist,
+                                   returnMatrix=FALSE)
 {
   if (!is.null(zmat)) {
     ncov   <- ncol(zmat)
@@ -520,8 +525,8 @@ fngrad.smooth.monotone <- function(yi, argvals, zmat, wtvec, cveci, lambda,
   n      <- length(argvals)
   nbasis <- basisobj$nbasis
   Wfdobj <- fd(cveci, basisobj)
-  h      <- monfn(argvals, Wfdobj, basislist)
-  Dyhat  <- mongrad(argvals, Wfdobj, basislist)
+  h      <- monfn(argvals, Wfdobj, basislist, returnMatrix)
+  Dyhat  <- mongrad(argvals, Wfdobj, basislist, returnMatrix)
   if (!is.null(zmat)) {
     xmat <- cbind(zmat,h)
   } else {
@@ -531,7 +536,7 @@ fngrad.smooth.monotone <- function(yi, argvals, zmat, wtvec, cveci, lambda,
   Dxmat[,ncovp1,] <- Dyhat
   wtroot <- sqrt(wtvec)
   wtrtmt <- wtroot %*% matrix(1,1,ncovp1)
-  yroot  <- yi*wtroot
+  yroot  <- yi*as.numeric(wtroot)
   xroot  <- xmat*wtrtmt
   #  compute regression coefs.
   betai  <- lsfit(xmat, yi, wt=as.vector(wtvec), intercept=FALSE)$coef
